@@ -5,6 +5,8 @@
  *
  * Copyright © 2008-2009 Michael Stapelberg and contributors
  * Copyright © 2009 Thorsten Toepper <atsutane at freethoughts dot de>
+ * Copyright © 2010 Axel Wagner <mail at merovius dot de>
+ * Copyright © 2010 Fernando Tarlá Cardoso Lemos <fernandotcl at gmail dot com>
  *
  * See file LICENSE for license information.
  *
@@ -27,6 +29,8 @@
 
 #include "i3status.h"
 
+#define exit_if_null(pointer, ...) { if (pointer == NULL) die(__VA_ARGS__); }
+
 /* socket file descriptor for general purposes */
 int general_socket;
 
@@ -43,24 +47,123 @@ void sigpipe(int signum) {
 }
 
 /*
- * Checks if there is a file at the given path (expanding ~) and returns the
- * full path if so or NULL if there is no file.
+ * Checks if the given path exists by calling stat().
  *
  */
-static char *file_exists(char *path) {
-        static glob_t globbuf;
+static bool path_exists(const char *path) {
         struct stat buf;
-        char *full_path = NULL;
+        return (stat(path, &buf) == 0);
+}
 
-        if (glob(path, GLOB_NOCHECK | GLOB_TILDE, NULL, &globbuf) < 0)
-                return NULL;
+static void *scalloc(size_t size) {
+        void *result = calloc(size, 1);
+        exit_if_null(result, "Error: out of memory (calloc(%zd))\n", size);
+        return result;
+}
 
-        full_path = (globbuf.gl_pathc > 0 ? globbuf.gl_pathv[0] : path);
+static char *sstrdup(const char *str) {
+        char *result = strdup(str);
+        exit_if_null(result, "Error: out of memory (strdup())\n");
+        return result;
+}
 
-        if (stat(full_path, &buf) < 0)
-                return NULL;
 
-        return full_path;
+/*
+ * Validates a color in "#RRGGBB" format
+ *
+ */
+static int valid_color(const char *value)
+{
+        if (strlen(value) != 7) return 0;
+        if (value[0] != '#') return 0;
+        for (int i = 1; i < 7; ++i) {
+                if (value[i] >= '0' && value[i] <= '9') continue;
+                if (value[i] >= 'a' && value[i] <= 'f') continue;
+                if (value[i] >= 'A' && value[i] <= 'F') continue;
+                return 0;
+        }
+        return 1;
+}
+
+/*
+ * This function resolves ~ in pathnames.
+ * It may resolve wildcards in the first part of the path, but if no match
+ * or multiple matches are found, it just returns a copy of path as given.
+ *
+ */
+static char *resolve_tilde(const char *path) {
+        static glob_t globbuf;
+        char *head, *tail, *result = NULL;
+
+        tail = strchr(path, '/');
+        head = strndup(path, tail ? (size_t)(tail - path) : strlen(path));
+
+        int res = glob(head, GLOB_TILDE, NULL, &globbuf);
+        free(head);
+        /* no match, or many wildcard matches are bad */
+        if (res == GLOB_NOMATCH || globbuf.gl_pathc != 1)
+                result = sstrdup(path);
+        else if (res != 0) {
+                die("glob() failed");
+        } else {
+                head = globbuf.gl_pathv[0];
+                result = scalloc(strlen(head) + (tail ? strlen(tail) : 0) + 1);
+                strncpy(result, head, strlen(head));
+                strncat(result, tail, strlen(tail));
+        }
+        globfree(&globbuf);
+
+        return result;
+}
+
+static char *get_config_path() {
+        /* 1: check for $XDG_CONFIG_HOME/i3/config */
+        char *xdg_config_home, *xdg_config_dirs, *config_path;
+
+        if ((xdg_config_home = getenv("XDG_CONFIG_HOME")) == NULL)
+                xdg_config_home = "~/.config";
+
+        xdg_config_home = resolve_tilde(xdg_config_home);
+        if (asprintf(&config_path, "%s/i3status/config", xdg_config_home) == -1)
+                die("asprintf() failed");
+        free(xdg_config_home);
+
+        if (path_exists(config_path))
+                return config_path;
+        free(config_path);
+
+        /* 2: check for $XDG_CONFIG_DIRS/i3/config */
+        if ((xdg_config_dirs = getenv("XDG_CONFIG_DIRS")) == NULL)
+                xdg_config_dirs = "/etc/xdg";
+
+        char *buf = strdup(xdg_config_dirs);
+        char *tok = strtok(buf, ":");
+        while (tok != NULL) {
+                tok = resolve_tilde(tok);
+                if (asprintf(&config_path, "%s/i3status/config", tok) == -1)
+                        die("asprintf() failed");
+                free(tok);
+                if (path_exists(config_path)) {
+                        free(buf);
+                        return config_path;
+                }
+                free(config_path);
+                tok = strtok(NULL, ":");
+        }
+        free(buf);
+
+        /* 3: check traditional paths */
+        config_path = resolve_tilde("~/.i3status.conf");
+        if (path_exists(config_path))
+                return config_path;
+
+        config_path = strdup(SYSCONFDIR "/i3status.conf");
+        if (!path_exists(config_path))
+                die("Neither $XDG_CONFIG_HOME/i3status/config, nor "
+                    "$XDG_CONFIG_DIRS/i3status/config, nor ~/.i3status.conf nor "
+                    SYSCONFDIR "/i3status.conf exist.");
+
+        return config_path;
 }
 
 int main(int argc, char *argv[]) {
@@ -69,6 +172,10 @@ int main(int argc, char *argv[]) {
         cfg_opt_t general_opts[] = {
                 CFG_STR("output_format", "dzen2", CFGF_NONE),
                 CFG_BOOL("colors", 1, CFGF_NONE),
+                CFG_STR("color_good", "#00FF00", CFGF_NONE),
+                CFG_STR("color_degraded", "#FFFF00", CFGF_NONE),
+                CFG_STR("color_bad", "#FF0000", CFGF_NONE),
+                CFG_STR("color_separator", "#333333", CFGF_NONE),
                 CFG_INT("interval", 1, CFGF_NONE),
                 CFG_END()
         };
@@ -80,7 +187,7 @@ int main(int argc, char *argv[]) {
         };
 
         cfg_opt_t wireless_opts[] = {
-                CFG_STR("format_up", "W: (%quality at %essid) %ip", CFGF_NONE),
+                CFG_STR("format_up", "W: (%quality at %essid, %bitrate) %ip", CFGF_NONE),
                 CFG_STR("format_down", "W: down", CFGF_NONE),
                 CFG_END()
         };
@@ -108,6 +215,11 @@ int main(int argc, char *argv[]) {
                 CFG_END()
         };
 
+        cfg_opt_t ddate_opts[] = {
+                CFG_STR("format", "%{%a, %b %d%}, %Y%N - %H", CFGF_NONE),
+                CFG_END()
+        };
+
         cfg_opt_t load_opts[] = {
                 CFG_STR("format", "%5min %10min %15min", CFGF_NONE),
                 CFG_END()
@@ -123,6 +235,14 @@ int main(int argc, char *argv[]) {
                 CFG_END()
         };
 
+        cfg_opt_t volume_opts[] = {
+                CFG_STR("format", "♪: %volume", CFGF_NONE),
+                CFG_STR("device", "default", CFGF_NONE),
+                CFG_STR("mixer", "Master", CFGF_NONE),
+                CFG_INT("mixer_idx", 0, CFGF_NONE),
+                CFG_END()
+        };
+
         cfg_opt_t opts[] = {
                 CFG_STR_LIST("order", "{ipv6,\"run_watch DHCP\",\"wireless wlan0\",\"ethernet eth0\",\"battery 0\",\"cpu_temperature 0\",load,time}", CFGF_NONE),
                 CFG_SEC("general", general_opts, CFGF_NONE),
@@ -132,17 +252,20 @@ int main(int argc, char *argv[]) {
                 CFG_SEC("battery", battery_opts, CFGF_TITLE | CFGF_MULTI),
                 CFG_SEC("cpu_temperature", temp_opts, CFGF_TITLE | CFGF_MULTI),
                 CFG_SEC("disk", disk_opts, CFGF_TITLE | CFGF_MULTI),
+                CFG_SEC("volume", volume_opts, CFGF_TITLE | CFGF_MULTI),
                 CFG_SEC("ipv6", ipv6_opts, CFGF_NONE),
                 CFG_SEC("time", time_opts, CFGF_NONE),
+                CFG_SEC("ddate", ddate_opts, CFGF_NONE),
                 CFG_SEC("load", load_opts, CFGF_NONE),
                 CFG_END()
         };
 
-        char *configfile;
+        char *configfile = NULL;
         int o, option_index = 0;
         struct option long_options[] = {
                 {"config", required_argument, 0, 'c'},
                 {"help", no_argument, 0, 'h'},
+                {"version", no_argument, 0, 'v'},
                 {0, 0, 0, 0}
         };
 
@@ -151,23 +274,21 @@ int main(int argc, char *argv[]) {
         action.sa_handler = sigpipe;
         sigaction(SIGPIPE, &action, NULL);
 
-        /* Figure out which configuration file to use before the user may
-         * override this setting using -c */
-
-        if ((configfile = file_exists("~/.i3status.conf")) == NULL)
-                configfile = file_exists(PREFIX "/etc/i3status.conf");
-
-        while ((o = getopt_long(argc, argv, "c:h", long_options, &option_index)) != -1)
+        while ((o = getopt_long(argc, argv, "c:hv", long_options, &option_index)) != -1)
                 if ((char)o == 'c')
                         configfile = optarg;
                 else if ((char)o == 'h') {
-                        printf("i3status © 2008-2009 Michael Stapelberg\n"
-                                "Syntax: %s [-c <configfile>]\n", argv[0]);
+                        printf("i3status " VERSION " © 2008-2010 Michael Stapelberg and contributors\n"
+                                "Syntax: %s [-c <configfile>] [-h] [-v]\n", argv[0]);
+                        return 0;
+                } else if ((char)o == 'v') {
+                        printf("i3status " VERSION " © 2008-2010 Michael Stapelberg and contributors\n");
                         return 0;
                 }
 
+
         if (configfile == NULL)
-                die("No configuration file found\n");
+                configfile = get_config_path();
 
         cfg = cfg_init(opts, CFGF_NONE);
         if (cfg_parse(cfg, configfile) == CFG_PARSE_ERROR)
@@ -186,6 +307,12 @@ int main(int argc, char *argv[]) {
                 output_format = O_NONE;
         else die("Unknown output format: \"%s\"\n", output_str);
 
+        if (!valid_color(cfg_getstr(cfg_general, "color_good"))
+                        || !valid_color(cfg_getstr(cfg_general, "color_degraded"))
+                        || !valid_color(cfg_getstr(cfg_general, "color_bad"))
+                        || !valid_color(cfg_getstr(cfg_general, "color_separator")))
+               die("Bad color format");
+
         if ((general_socket = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
                 die("Could not create socket\n");
 
@@ -222,6 +349,15 @@ int main(int argc, char *argv[]) {
                         CASE_SEC("time")
                                 print_time(cfg_getstr(sec, "format"));
 
+                        CASE_SEC("ddate")
+                                print_ddate(cfg_getstr(sec, "format"));
+
+                        CASE_SEC_TITLE("volume")
+                                print_volume(cfg_getstr(sec, "format"),
+                                             cfg_getstr(sec, "device"),
+                                             cfg_getstr(sec, "mixer"),
+                                             cfg_getint(sec, "mixer_idx"));
+
                         CASE_SEC_TITLE("cpu_temperature")
                                 print_cpu_temperature_info(atoi(title), cfg_getstr(sec, "format"));
                 }
